@@ -3,7 +3,7 @@
 package App::WHMCSUtils;
 
 our $DATE = '2019-09-19'; # DATE
-our $VERSION = '0.006'; # VERSION
+our $VERSION = '0.007'; # VERSION
 
 use 5.010001;
 use strict;
@@ -721,6 +721,8 @@ sub _login_admin {
 sub _send_verification_email {
     my ($args, $client_rec) = @_;
 
+    _login_admin(%$args);
+
     my $url0 = "$args->{url}/admin/clientssummary.php";
     my $url1 = "$url0?userid=$client_rec->{id}";
     $mech->get($url1);
@@ -775,6 +777,35 @@ _
             schema => ['array*', of=>'uint*', 'x.perl.coerce_rules'=>['str_comma_sep']],
             tags => ['category:filtering'],
         },
+        include_active => {
+            summary => 'Whether to include active clients',
+            schema => ['bool*'],
+            default => 1,
+            tags => ['category:filtering'],
+        },
+        include_inactive => {
+            summary => 'Whether to include inactive clients',
+            schema => ['bool*'],
+            default => 0,
+            tags => ['category:filtering'],
+        },
+        hook_set_sender_email => {
+            summary => 'Hook to set sender email for every email',
+            description => <<'_',
+
+Hook will receive these arguments:
+
+    ($client_rec, $orig_sender_email)
+
+`$client_rec` is a hash containing client record fields, e.g. `id`, `email`,
+`firstname`, `lastname`, etc. `$orig_sender_email` is the original sender email
+setting (`Email` setting in the configuration table).
+
+Hook is expected to return the sender email.
+
+_
+            schema => ['any*', of=>['str*', 'code*']],
+        },
     },
     features => {
         dry_run => 1,
@@ -789,8 +820,10 @@ sub send_verification_emails {
     my $sth = $dbh->prepare(
         join("",
              "SELECT id,firstname,lastname,companyname,email FROM tblclients ",
-             "WHERE status='Active' AND email_verified=0",
-             ($args{include_client_ids} ? " AND id IN (".join(",",map{$_+0} @{ $args{include_client_ids} }).")" : ""),
+             "WHERE email_verified=0 ",
+             (defined $args{include_active}   && !$args{include_active}   ? "" : "AND status <> 'Active' "  ),
+             (defined $args{include_inactive} && !$args{include_inactive} ? "" : "AND status <> 'Inactive' "),
+             ($args{include_client_ids} ? "AND id IN (".join(",",map{$_+0} @{ $args{include_client_ids} }).")" : ""),
              "ORDER BY ".($args{random} ? "RAND()" : "id"),
          ),
     );
@@ -801,20 +834,38 @@ sub send_verification_emails {
     while (my $row = $sth->fetchrow_hashref) {
         push @client_recs, $row;
     }
+    log_info "Found %d client email(s)", scalar(@client_recs);
 
     if ($args{action} eq 'list-clients') {
         return [200, "OK", \@client_recs];
     }
 
     my $i = 0;
-    _login_admin(%args);
+    my ($orig_sender_email) = $dbh->selectrow_array("SELECT value FROM tblconfiguration WHERE setting='Email'");
+
     for my $client_rec (@client_recs) {
         $i++;
-        log_info "[%d/%d] Sending verification email for client #%d (%s %s, email %s) ...",
+        my $sender_email = $orig_sender_email;
+        if ($args{hook_set_sender_email}) {
+            unless (ref $args{hook_set_sender_email} eq 'CODE') {
+                $args{hook_set_sender_email} = eval "sub { $args{hook_set_sender_email} }";
+                die "Can't compile code in hook_set_sender_email: $@" if $@;
+            }
+            $sender_email = $args{hook_set_sender_email}->($client_rec, $orig_sender_email);
+            $dbh->do("UPDATE tblconfiguration SET value=? WHERE setting='Email'", {}, $sender_email);
+        }
+        log_info "[%d/%d]%s Sending verification email (sender email %s) for client #%d (%s %s, email %s) ...",
             $i, scalar(@client_recs),
+            $args{-dry_run} ? " [DRY-RUN]" : "",
+            $sender_email,
             $client_rec->{id}, $client_rec->{firstname}, $client_rec->{lastname}, $client_rec->{email};
+        next if $args{-dry_run};
         _send_verification_email(\%args, $client_rec);
     }
+
+    # XXX execute even when we're SIGINT'ed
+    $dbh->do("UPDATE tblconfiguration SET value=? WHERE setting='Email'", {}, $orig_sender_email);
+
     [200];
 }
 
@@ -833,7 +884,7 @@ App::WHMCSUtils - CLI utilities related to WHMCS
 
 =head1 VERSION
 
-This document describes version 0.006 of App::WHMCSUtils (from Perl distribution App-WHMCSUtils), released on 2019-09-19.
+This document describes version 0.007 of App::WHMCSUtils (from Perl distribution App-WHMCSUtils), released on 2019-09-19.
 
 =head1 FUNCTIONS
 
@@ -1058,8 +1109,30 @@ clients who haven't got their email verified yet.
 
 =item * B<db_user> => I<str>
 
+=item * B<hook_set_sender_email> => I<str|code>
+
+Hook to set sender email for every email.
+
+Hook will receive these arguments:
+
+ ($client_rec, $orig_sender_email)
+
+C<$client_rec> is a hash containing client record fields, e.g. C<id>, C<email>,
+C<firstname>, C<lastname>, etc. C<$orig_sender_email> is the original sender email
+setting (C<Email> setting in the configuration table).
+
+Hook is expected to return the sender email.
+
+=item * B<include_active> => I<bool> (default: 1)
+
+Whether to include active clients.
+
 =item * B<include_client_ids> => I<array[uint]>
 
+=item * B<include_inactive> => I<bool> (default: 0)
+
+Whether to include inactive clients.
+
 =item * B<mech_user_agent> => I<str>
 
 =item * B<random> => I<bool> (default: 1)
